'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var test = _interopDefault(require('tape'));
var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));
var ReactTestUtils = _interopDefault(require('react-dom/test-utils'));
var Styletron = _interopDefault(require('styletron-server'));
var styletronUtils = require('styletron-utils');

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var STYLETRON_KEY = '__STYLETRON';

/**
 * Helper function to create styled element components
 * @packagename styletron-react
 * @param  {String|function} base        Tag name or component
 * @param  {function|object} styleFn     Style object or function that returns a style object
 * @param  {function}        assignProps Function that consumes the style result and props and returns an object with new props
 * @return {function}                    Component
 * @example
 */
function core(base, style, assignProps) {
  if (typeof base === 'function' && base[STYLETRON_KEY]) {
    var _base$STYLETRON_KEY = base[STYLETRON_KEY],
        tag = _base$STYLETRON_KEY.tag,
        styles = _base$STYLETRON_KEY.styles;
    // Styled component

    return createStyledElementComponent(tag, styles.concat(style), assignProps);
  }
  if (typeof base === 'string' || typeof base === 'function') {
    // Tag name or non-styled component
    return createStyledElementComponent(base, [style], assignProps);
  }
  throw new Error('`styled` takes either a DOM element name or a component');
}

function createStyledElementComponent(base, stylesArray, assignProps) {
  function StyledElement(props, context) {
    var ownProps = assign({}, props);
    delete ownProps.innerRef;

    var styleResult = {};
    StyledElement[STYLETRON_KEY].styles.forEach(function (style) {
      if (typeof style === 'function') {
        assign(styleResult, style(ownProps, context));
      } else if ((typeof style === 'undefined' ? 'undefined' : _typeof(style)) === 'object') {
        assign(styleResult, style);
      }
    });

    var elementProps = assignProps(context.styletron, styleResult, ownProps);

    if (props.innerRef) {
      elementProps.ref = props.innerRef;
    }

    return React.createElement(StyledElement[STYLETRON_KEY].tag, elementProps);
  }

  StyledElement[STYLETRON_KEY] = {
    tag: base,
    styles: stylesArray
  };

  StyledElement.contextTypes = { styletron: PropTypes.object };

  if (process.env.NODE_ENV !== 'production') {
    var name = base.displayName ? base.displayName : typeof base === 'function' ? base.name : base;
    StyledElement.displayName = 'Styled' + (name ? '(' + name + ')' : '');
  }

  return StyledElement;
}

function assign(target, source) {
  for (var key in source) {
    target[key] = source[key];
  }
  return target;
}

/**
 * Helper function to create styled element components
 * @packagename styletron-react
 * @param  {String|function} base     Tag name or styled element component
 * @param  {function|object} styleFn  Style object or function that returns a style object
 * @return {function}                 Styled element component
 * @example
 * import {styled} from 'styletron-react';
 *
 * const Panel = styled('div', {
 *   backgroundColor: 'lightblue',
 *   fontSize: '12px'
 * });
 *
 * <Panel>Hello World</Panel>
 * @example
 * import {styled} from 'styletron-react';
 *
 * const Panel = styled('div', (props) => ({
 *   backgroundColor: props.alert ? 'orange' : 'lightblue',
 *   fontSize: '12px'
 * }));
 *
 * <Panel alert>Danger!</Panel>
 * @example
 * import {styled} from 'styletron-react';
 *
 * const DeluxePanel = styled(Panel, (props) => ({
 *   backgroundColor: props.alert ? 'firebrick' : 'rebeccapurple',
 *   color: 'white',
 *   boxShadow: '3px 3px 3px darkgray'
 * }));
 *
 * <DeluxePanel>Bonjour Monde</DeluxePanel>
 */
function styled(base, style) {
  return core(base, style, assignProps);
}

function assignProps(styletron, styleResult, ownProps) {
  var styletronClassName = styletronUtils.injectStylePrefixed(styletron, styleResult);
  // Skipping cloning of `ownProps` since that's already done internally
  if (ownProps.styleProps) {
    delete ownProps.styleProps;
  }
  ownProps.className = ownProps.className ? ownProps.className + ' ' + styletronClassName : styletronClassName;
  return ownProps;
}

function _classCallCheck$1(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$1(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$1(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @class StyletronProvider
 * @packagename styletron-react
 * @description Provides a Styletron instance to descendant styled components via context
 * @example
 * const Styletron = require('styletron');
 *
 * function render() {
 *   return React.renderToString(
 *     <StyletronProvider styletron={new Styletron()}>
 *       <App/>
 *     </StyletronProvider>
 *   );
 * }
 * 
 * @property {object} styletron - Styletron instance
 * @property {ReactElement} children - children
 * @extends ReactClass
 */

var StyletronProvider = function (_React$Component) {
  _inherits$1(StyletronProvider, _React$Component);

  StyletronProvider.prototype.getChildContext = function getChildContext() {
    return { styletron: this.styletron };
  };

  function StyletronProvider(props, context) {
    _classCallCheck$1(this, StyletronProvider);

    var _this = _possibleConstructorReturn$1(this, _React$Component.call(this, props, context));

    _this.styletron = props.styletron;
    return _this;
  }

  StyletronProvider.prototype.render = function render() {
    return React.Children.only(this.props.children);
  };

  return StyletronProvider;
}(React.Component);

StyletronProvider.PropTypes = {
  styletron: PropTypes.object.isRequired,
  children: PropTypes.element.isRequired
};

StyletronProvider.childContextTypes = {
  styletron: PropTypes.object.isRequired
};

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/* global process */
/* eslint-env browser */

function strictAssignProps(styletron, styleResult, ownProps) {
  return Object.assign({}, ownProps, {
    className: styletronUtils.injectStylePrefixed(styletron, styleResult)
  });
}

test('provider provides instance', function (t) {
  var mockInstance = {};
  var MockComponent = function MockComponent(props, context) {
    t.equal(context.styletron, mockInstance, 'styletron instance override provided');
    return React.createElement('div');
  };
  MockComponent.contextTypes = { styletron: PropTypes.object };
  ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, {
    styletron: mockInstance
  }, React.createElement(MockComponent)));
  t.end();
});

test('props passed to core function', function (t) {
  t.plan(1);
  var expected = {
    prop1: 'foo'
  };
  var Widget = core('div', function (props) {
    t.deepEqual(props, expected, 'props accessible in core fn');
    return {};
  }, strictAssignProps);
  var styletron = new Styletron();
  ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron: styletron }, React.createElement(Widget, expected)));
});

test('core applies styles', function (t) {
  var Widget = core('div', function () {
    return { color: 'red' };
  }, strictAssignProps);
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron: styletron }, React.createElement(Widget)));
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a', 'styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}');
  t.end();
});

test('core applies static styles', function (t) {
  var Widget = core('div', { color: 'red' }, strictAssignProps);
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron: styletron }, React.createElement(Widget)));
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a', 'matches expected styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}');
  t.end();
});

test('core passes through valid props', function (t) {
  var Widget = core('div', { color: 'red' }, strictAssignProps);
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron: styletron }, React.createElement(Widget, {
    'data-bar': 'bar'
  })));
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.getAttribute('data-bar'), 'bar', 'valid attribute prop passed through');
  t.end();
});

test('core composition', function (t) {
  var Widget = core('div', { color: 'red', display: 'inline' }, strictAssignProps);
  var SuperWidget = core(Widget, { display: 'block', background: 'black' }, strictAssignProps);
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron: styletron }, React.createElement(SuperWidget)));
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a b c', 'matches expected styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}.b{display:block}.c{background:black}');
  t.end();
});

test('core component', function (t) {
  var Widget = function Widget(_ref) {
    var className = _ref.className;
    return React.createElement('div', { className: className });
  };
  var SuperWidget = core(Widget, { color: 'red' }, strictAssignProps);
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron: styletron }, React.createElement(SuperWidget)));
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a', 'matches expected styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}');
  t.end();
});

test('innerRef works', function (t) {
  t.plan(1);

  var Widget = core('button', { color: 'red' }, strictAssignProps);
  var styletron = new Styletron();

  var TestComponent = function (_React$Component) {
    _inherits(TestComponent, _React$Component);

    function TestComponent() {
      _classCallCheck(this, TestComponent);

      return _possibleConstructorReturn(this, _React$Component.apply(this, arguments));
    }

    TestComponent.prototype.componentDidMount = function componentDidMount() {
      t.ok(this.widgetInner instanceof HTMLButtonElement, 'is button');
    };

    TestComponent.prototype.render = function render() {
      var _this2 = this;

      return React.createElement(Widget, {
        innerRef: function innerRef(c) {
          _this2.widgetInner = c;
        }
      });
    };

    return TestComponent;
  }(React.Component);

  ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron: styletron }, React.createElement(TestComponent)));
});

test('innerRef not passed', function (t) {
  t.plan(2);

  var InnerComponent = function (_React$Component2) {
    _inherits(InnerComponent, _React$Component2);

    function InnerComponent() {
      _classCallCheck(this, InnerComponent);

      return _possibleConstructorReturn(this, _React$Component2.apply(this, arguments));
    }

    InnerComponent.prototype.render = function render() {
      t.deepEqual(this.props, {
        className: 'a',
        foo: 'bar'
      }, 'props match expected');
      return React.createElement(
        'button',
        null,
        'InnerComponent'
      );
    };

    return InnerComponent;
  }(React.Component);

  var Widget = core(InnerComponent, { color: 'red' }, strictAssignProps);
  var styletron = new Styletron();

  var TestComponent = function (_React$Component3) {
    _inherits(TestComponent, _React$Component3);

    function TestComponent() {
      _classCallCheck(this, TestComponent);

      return _possibleConstructorReturn(this, _React$Component3.apply(this, arguments));
    }

    TestComponent.prototype.componentDidMount = function componentDidMount() {
      t.ok(ReactTestUtils.isCompositeComponentWithType(this.widgetInner, InnerComponent), 'is InnerComponent');
    };

    TestComponent.prototype.render = function render() {
      var _this5 = this;

      return React.createElement(Widget, {
        foo: 'bar',
        innerRef: function innerRef(c) {
          _this5.widgetInner = c;
        }
      });
    };

    return TestComponent;
  }(React.Component);

  ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron: styletron }, React.createElement(TestComponent)));
});

test('styleProps not passed', function (t) {
  t.plan(1);

  var InnerComponent = function (_React$Component4) {
    _inherits(InnerComponent, _React$Component4);

    function InnerComponent() {
      _classCallCheck(this, InnerComponent);

      return _possibleConstructorReturn(this, _React$Component4.apply(this, arguments));
    }

    InnerComponent.prototype.render = function render() {
      t.deepEqual(this.props, {
        className: 'a',
        'data-foo': 'bar'
      }, 'props match expected');
      return React.createElement(
        'button',
        null,
        'InnerComponent'
      );
    };

    return InnerComponent;
  }(React.Component);

  var Widget = styled(InnerComponent, { color: 'red' });
  var styletron = new Styletron();

  ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron: styletron }, React.createElement(Widget, {
    'data-foo': 'bar',
    styleProps: {
      baz: 'qux'
    }
  })));
});

test('styled merges class name prop', function (t) {
  var Widget = styled('div', { color: 'red' });
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(React.createElement(StyletronProvider, { styletron: styletron }, React.createElement(Widget, { className: 'foo' })));
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'foo a', 'matches expected classes');
  t.end();
});

test('core sets display name', function (t) {
  var env = process.env.NODE_ENV;

  function developmentDisplayName(createComponent) {
    process.env.NODE_ENV = 'development';
    var displayName = createComponent().displayName;
    process.env.NODE_ENV = env;
    return displayName;
  }
  function productionDisplayName(createComponent) {
    process.env.NODE_ENV = 'production';
    var displayName = createComponent().displayName;
    process.env.NODE_ENV = env;
    return displayName;
  }

  t.equal(developmentDisplayName(function () {
    return styled('div', { color: 'red' });
  }), 'Styled(div)', 'matches expected display name');
  t.equal(productionDisplayName(function () {
    return styled('div', { color: 'red' });
  }), void 0, 'matches expected display name');

  var TestComponentClass = function (_React$Component5) {
    _inherits(TestComponentClass, _React$Component5);

    function TestComponentClass() {
      _classCallCheck(this, TestComponentClass);

      return _possibleConstructorReturn(this, _React$Component5.apply(this, arguments));
    }

    TestComponentClass.prototype.render = function render() {
      return React.createElement('div', null);
    };

    return TestComponentClass;
  }(React.Component);

  t.equal(developmentDisplayName(function () {
    return styled(TestComponentClass, { color: 'red' });
  }), 'Styled(TestComponentClass)', 'matches expected display name');
  t.equal(productionDisplayName(function () {
    return styled(TestComponentClass, { color: 'red' });
  }), void 0, 'matches expected display name');

  TestComponentClass.displayName = 'Test';

  t.equal(developmentDisplayName(function () {
    return styled(TestComponentClass, { color: 'red' });
  }), 'Styled(Test)', 'matches expected display name');
  t.equal(productionDisplayName(function () {
    return styled(TestComponentClass, { color: 'red' });
  }), void 0, 'matches expected display name');

  function TestStatelessComponent() {
    return React.createElement('div', null);
  }

  t.equal(developmentDisplayName(function () {
    return styled(TestStatelessComponent, { color: 'red' });
  }), 'Styled(TestStatelessComponent)', 'matches expected display name');
  t.equal(productionDisplayName(function () {
    return styled(TestStatelessComponent, { color: 'red' });
  }), void 0, 'matches expected display name');

  TestStatelessComponent.displayName = 'Test';

  t.equal(developmentDisplayName(function () {
    return styled(TestStatelessComponent, { color: 'red' });
  }), 'Styled(Test)', 'matches expected display name');
  t.equal(productionDisplayName(function () {
    return styled(TestStatelessComponent, { color: 'red' });
  }), void 0, 'matches expected display name');

  t.end();
});
//# sourceMappingURL=browser.js.map
